/*
 * post_raw.h
 * Main header file for libpost_raw
 * listpostal project
 * libpost_raw library
 * $Id: post_raw.h,v 1.10 2001/05/03 12:27:55 fullermd Exp $
 */


/* Header files including header files.  Bad monkey! */
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>


/* Typedef */
/* Generic */
typedef struct postal_conn POSTAL_CONN;
typedef struct postal_msg POSTAL_MSG;
typedef struct postal_msg_list POSTAL_MSG_LIST;
/* POP specific */
typedef struct spec_pop SPEC_POP;
typedef struct spec_mbox SPEC_MBOX;
typedef struct pop_ret_stat POP_RET_STAT;
typedef struct pop_ret_list POP_RET_LIST;


/* Function prototypes */
/* Generic */
void postal_free_msg(POSTAL_MSG *);
void postal_free_msglist(POSTAL_MSG_LIST *);

/* mbox */
POSTAL_CONN *mbox_open_file(const char *, int);
void mbox_write_msg(POSTAL_CONN *, POSTAL_MSG *);
void mbox_close_file(POSTAL_CONN *);

/* POP */
int pop_close(POSTAL_CONN *);
POSTAL_CONN *pop_connect(const char *, const char *, const char *);
int pop_dele(POSTAL_CONN *, int);
void pop_free_retlist(POP_RET_LIST *);
POP_RET_LIST *pop_list(POSTAL_CONN *);
int pop_login(POSTAL_CONN *, const char *, const char *);
POSTAL_CONN *pop_network_connect(struct sockaddr_in);
char *pop_retr(POSTAL_CONN *, int);
POSTAL_MSG *pop_retr_parse(POSTAL_CONN *, int);
POSTAL_MSG_LIST *pop_retr_all(POSTAL_CONN *, int);
POP_RET_STAT *pop_stat(POSTAL_CONN *);


/* Macros */
#define PCONN_SIZE		(sizeof(POSTAL_CONN))
#define SPOP_SIZE		(sizeof(SPEC_POP))
#define SMBOX_SIZE		(sizeof(SPEC_MBOX))
#define PRSTAT_SIZE		(sizeof(POP_RET_STAT))
#define PRLIST_SIZE		(sizeof(POP_RET_LIST))
#define PMSG_SIZE		(sizeof(POSTAL_MSG))
#define PMSGLIST_SIZE	(sizeof(POSTAL_MSG_LIST))


/* Information access macros */
/* Generic */
#define postal_msg_header(prs)		(prs)->header
#define postal_msg_body(prs)		(prs)->body
#define postal_mlist_msg(prs)		(prs)->msg
#define postal_mlist_next(prs)		(prs)->next
/* POP */
#define pop_stat_msgs(prs)		(prs)->num_msg
#define pop_stat_bytes(prs)		(prs)->num_bytes
#define pop_list_mnum(prs)		(prs)->msg_num
#define pop_list_msize(prs)		(prs)->msg_size
#define pop_list_mnext(prs)		(prs)->next


/* Structures */
/* postal_conn: A single libpost_raw connection */
struct postal_conn
{
	union
	{
		SPEC_POP	*pop;
		SPEC_MBOX	*mbox;
	} spec;
};


/* spec_mbox: Special info for mbox connections */
struct spec_pop
{
	int		sock;
	char	*srv_header;		/* Server greeting header */
};


/* spec_mbox: Special info for mbox's */
struct spec_mbox
{
	char	*filename;			/* Filename */
	union
	{
		FILE	*mbox_file;		/* stdio file descriptor */
		int		mbox_desc;		/* raw file descriptor */
	} fref;
};


/* pop_ret_stat: Information return from POP STAT query */
struct pop_ret_stat
{
	unsigned int num_msg;
	unsigned long num_bytes;
};


/* pop_ret_list: Individual 'list' message */
struct pop_ret_list
{
	unsigned int msg_num;
	unsigned long msg_size;
	POP_RET_LIST *next;
};

/* postal_msg: Parsed out full email message */
struct postal_msg
{
	char *header;
	char *body;
};

/* postal_msg_list : List of parsed messages */
struct postal_msg_list
{
	POSTAL_MSG *msg;
	POSTAL_MSG_LIST *next;
};
