/*
 * pop_get.c
 * Sample POP3 -> mbox program
 * Links again: libpost_raw
 * Links shared
 * $Id: pop_get.c,v 1.2 2001/05/03 15:03:21 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "../../include/post_raw/post_raw.h"


int
main(int argc, char *argv[])
{
	POSTAL_CONN *pop_conn, *mbox_conn;
	POSTAL_MSG_LIST *ret_head, *ret_tmp;
	int dele=0;
	int goch;
	char *p_serv=NULL, *p_user=NULL, *p_pass=NULL, *f_name=NULL;

	while ( (goch = getopt(argc, argv, "dh:u:p:m:")) != -1 )
		switch((char)goch)
		{
			case 'd':
				/* Do DELE's */
				dele = 1;
				break;
			case 'h':
				/* Hostname */
				p_serv = optarg;
				break;
			case 'u':
				/* Username */
				p_user = optarg;
				break;
			case 'p':
				/* Password */
				p_pass = optarg;
				break;
			case 'm':
				/* Filename */
				f_name = optarg;
				break;
			default:
				printf("Bad argument '%c'\n", goch);
				exit(1);
				/* NOTREACHED */
		}

	argc -= optind;
	argv += optind;

	/* Grab any non-command-line spec'd options interactively */
	if(!f_name)
	{
		printf("Enter mbox filename: ");
		fflush(stdout);
		f_name = malloc(1024);
		scanf("%s", f_name);
	}
	if(!p_serv)
	{
		printf("Enter hostname to fetch from: ");
		fflush(stdout);
		p_serv = malloc(1024);
		scanf("%s", p_serv);
	}
	if(!p_user)
	{
		printf("Enter username: ");
		fflush(stdout);
		p_user = malloc(1024);
		scanf("%s", p_user);
	}
	if(!p_pass)
	{
		p_pass = getpass("Enter password: ");
	}

	printf("Running with host '%s', user '%s', and pass '%s', "
			"writing into file %s\n",
			p_serv, p_user, p_pass, f_name);

	pop_conn = pop_connect(p_serv, p_user, p_pass);

	if(!pop_conn)
	{
		printf("You lose, too bad!\n");
		exit(1);
	}

	/* Get all the messages at once with pop_retr_all() */
	ret_head = pop_retr_all(pop_conn, dele);

	/* Open up the mbox */
	mbox_conn = mbox_open_file(f_name, 0);

	/* Write out the messages */
	for(ret_tmp = ret_head ; ret_tmp ; ret_tmp = postal_mlist_next(ret_tmp))
	{
		mbox_write_msg(mbox_conn, postal_mlist_msg(ret_tmp));
	}

	/* Free things up */
	postal_free_msglist(ret_head);

	mbox_close_file(mbox_conn);
	pop_close(pop_conn);

	/*
	 * Note that this has potential memory leaks in
	 * this program (not the library, at least that I know of)
	 * with the p_* variables and the f_name variable.
	 * Don't really feel like making up flexible solutions
	 * to that, for a sample program.
	 */

	exit(0);
}
