/*
 * mbox_open_file.c
 * Open a file to use as a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_open_file.c,v 1.5 2001/05/03 14:38:51 fullermd Exp $
 */

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>


/*
 * XXX Note:
 * For the moment, we only do flock()-style locking.
 * This should probably change to at least allow dot-locking
 * as well, common as that is.  I'm adding a locktype
 * argument for now, that's not used.  FIX THIS LATER!
 */
POSTAL_CONN *
mbox_open_file(const char *fname, int locktype)
{
	POSTAL_CONN *toret;
	struct stat st_ret;
	int from_str;
	int i;

	/* Allocate memory */
	toret = malloc(PCONN_SIZE);
	toret->spec.mbox = malloc(SMBOX_SIZE);

	/* Copy the filename into the SPEC_ */
	toret->spec.mbox->filename = malloc( (strlen(fname)+1) );
	strcpy(toret->spec.mbox->filename, fname);

	/* Open the file as a raw I/O stream. */
	toret->spec.mbox->fref.mbox_desc = open( fname,
			O_RDWR | O_APPEND | O_CREAT);

	/*
	 * flock() it.  We'll have to support other locking
	 * methods later, but...
	 * BSD has a O_EXLOCK flag to open() to flock() it
	 * them, but apparently Linux doesn't.  Sigh.
	 */
	flock(toret->spec.mbox->fref.mbox_desc, LOCK_EX);

	/* Set the permissions in a portable way */
	fchmod(toret->spec.mbox->fref.mbox_desc, S_IRUSR | S_IWUSR);

	/* Get the stat info */
	fstat(toret->spec.mbox->fref.mbox_desc, &st_ret);

	/* Get us a stdio stream, it's more fun */
	toret->spec.mbox->fref.mbox_file = fdopen(
			toret->spec.mbox->fref.mbox_desc, "a+" );

	/*
	 * Figure out if we already have a blank line at EOF.  
	 * I *THINK* we always should, and we'll always put one
	 * ourselves, but better safe etc.
	 */
	/* If we created a new file, we can skip all this */
	if( st_ret.st_size > 0 )
	{
		/* XXX Do this twice, to make sure we have \n\n at EOF */
		for(i=0;i<=1;i++)
		{
			fseek(toret->spec.mbox->fref.mbox_file, -1L, SEEK_END);
			from_str = getc(toret->spec.mbox->fref.mbox_file);
			if( ((char)from_str) != '\n')
			{
				/*
				 * Double check, just in case we've got a stupid file.
				 * If we do, we'd need more code to deal with it in
				 * a totally 'proper' manner, unfortunately,
				 * but I'm not writing that right now.
				 */
				if( ((char)from_str) != '\r')
				{
					ungetc(from_str, toret->spec.mbox->fref.mbox_file);
					/* Go to EOF and add a newline */
					fseek(toret->spec.mbox->fref.mbox_file, 0, SEEK_END);
					putc('\n', toret->spec.mbox->fref.mbox_file);
					fflush(toret->spec.mbox->fref.mbox_file);
				}
			}
			else
				fseek(toret->spec.mbox->fref.mbox_file, 0, SEEK_END);
		}
	}


	/* Return the structure, we're ready to rock */
	return(toret);
}
